# -*- coding: utf-8 -*-

from typing import Callable, Optional, Union
from dataclasses import dataclass

from qtpy import QtCore, QtGui

SMALL_COL_WIDTH = 120
MEDIUM_COL_WIDTH = 200

ALIGN_LEFT = int(QtCore.Qt.AlignVCenter | QtCore.Qt.AlignLeft)
ALIGN_RIGHT = int(QtCore.Qt.AlignVCenter | QtCore.Qt.AlignRight)
ALIGN_CENTER = int(QtCore.Qt.AlignVCenter | QtCore.Qt.AlignCenter)


@dataclass(frozen=True)
class ColumnItem:
    """Determines how an object attribute is shown."""

    name: str
    label: Optional[Callable]
    checkstate: Optional[Callable] = None
    doc: str = "<no help available>"
    col_visible: bool = True
    width: int = SMALL_COL_WIDTH
    alignment: Optional[Union[Callable, int]] = None
    line_wrap: str = "none"
    foreground_color: Optional[Union[Callable, str]] = None
    background_color: Optional[Union[Callable, str]] = None
    decoration: Optional[Union[Callable, QtGui.QIcon]] = None
    font: Optional[Union[Callable, QtGui.QFont]] = None

    def get_name(self):
        return self.name

    def get_label(self, tree_item):
        if self.label is None:
            return ""
        elif callable(self.label):
            return self.label(tree_item)
        return self.label

    def get_checkstate(self, tree_item):
        if self.checkstate is None:
            return None
        elif callable(self.checkstate):
            return self.checkstate(tree_item)
        return self.checkstate

    def get_font(self, tree_item):
        if self.font is None:
            return None
        elif callable(self.font):
            return self.font(tree_item)
        return self.font

    def get_foreground_color(self, tree_item):
        if self.foreground_color is None:
            return None
        elif callable(self.foreground_color):
            return self.foreground_color(tree_item)
        return self.foreground_color

    def get_background_color(self, tree_item):
        if self.background_color is None:
            return None
        elif callable(self.background_color):
            return self.background_color(tree_item)
        return self.background_color

    def get_decoration(self, tree_item):
        if self.decoration is None:
            return None
        elif callable(self.decoration):
            return self.decoration(tree_item)
        return self.decoration

    def get_alignment(self, tree_item):
        if self.alignment is None:
            return ALIGN_LEFT
        elif callable(self.alignment):
            return self.alignment(tree_item)
        return self.alignment



# -*- coding: utf-8 -*-

from typing import List, Union, Optional, Type
from qtpy import QtWidgets
from prettyqt import custom_validators, widgets
from prettyqt.utils import helpers


class ListInput(widgets.LineEdit):
    def __init__(
        self,
        default_value: Union[List[float], str] = "",
        typ: Type = int,
        allow_single: bool = False,
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        super().__init__(parent=parent)
        if typ is int:
            val = custom_validators.IntListValidator(allow_single=allow_single)
        elif typ is float:
            val = custom_validators.FloatListValidator(allow_single=allow_single)
        else:
            raise ValueError(f"Invalid type {typ}")
        self.set_validator(val)
        self.set_value(default_value)

    def get_value(self) -> List[float]:  # type: ignore[override]
        return helpers.string_to_num_array(self.text())

    def set_value(self, value: Union[List[float], str]):
        if isinstance(value, list):
            value = str(value)[1:-1].replace(" ", "")
        self.set_text(value)


if __name__ == "__main__":
    app = widgets.app()
    widget = ListInput()
    widget.show()
    app.exec_()
    print(widget.get_value())



# -*- coding: utf-8 -*-

from typing import Union, Tuple, List

from qtpy import QtGui, QtCore

from prettyqt import core


class Polygon(QtGui.QPolygon):
    def __len__(self) -> int:
        return self.size()

    def __bool__(self):
        return not self.isEmpty()

    def __contains__(self, point: QtCore.QPoint) -> bool:
        return self.containsPoint(point, QtCore.Qt.OddEvenFill)

    def __getitem__(self, index: int) -> core.Point:
        return core.Point(self.point(index))

    def __setitem__(self, index: int, value: Union[QtCore.QPoint, Tuple[int, int]]):
        if isinstance(value, tuple):
            self.setPoint(index, *value)
        else:
            self.setPoint(index, value)

    def __sub__(self, other: QtGui.QPolygon) -> "Polygon":
        return Polygon(self.subtracted(other))

    def __and__(self, other: QtGui.QPolygon) -> "Polygon":  # &
        return Polygon(self.intersected(other))

    def __xor__(self, other: QtGui.QPolygon) -> "Polygon":  # ^
        union = self | other
        intersect = self & other
        return union - intersect

    def __or__(self, other: QtGui.QPolygon) -> "Polygon":  # |
        return Polygon(self.united(other))

    def __reduce__(self):
        return type(self), (), self.__getstate__()

    def __getstate__(self):
        return core.DataStream.create_bytearray(self)

    def __setstate__(self, ba):
        core.DataStream.write_bytearray(ba, self)

    def get_point(self, index: int) -> core.Point:
        return core.Point(self.point(index))

    def get_points(self) -> List[core.Point]:
        return [self.get_point(i) for i in range(len(self))]

    def add_points(self, *points: Union[Tuple[float, float], core.Point]):
        for p in points:
            if isinstance(p, tuple):
                p = core.Point(*p)
            self << p

    @classmethod
    def from_xy(cls, xdata, ydata):
        import numpy as np

        size = len(xdata)
        polyline = cls(size)
        pointer = polyline.data()
        dtype, tinfo = np.float, np.finfo  # integers: = np.int, np.iinfo
        pointer.setsize(2 * polyline.size() * tinfo(dtype).dtype.itemsize)
        memory = np.frombuffer(pointer, dtype)
        memory[: (size - 1) * 2 + 1 : 2] = xdata
        memory[1 : (size - 1) * 2 + 2 : 2] = ydata
        return polyline


if __name__ == "__main__":
    poly = Polygon((core.Point(1, 1), core.Point(2, 2)))
    poly2 = Polygon((core.Point(1, 1), core.Point(2, 2)))
    poly & poly2



from qtpy import QtCore

from prettyqt import core


class Line(QtCore.QLine):
    def __getitem__(self, index):
        if index == 0:
            return core.Point(self.p1())
        elif index == 1:
            return core.Point(self.p2())

    def __setitem__(self, index, value):
        if index == 0:
            self.setP1(value)
        elif index == 1:
            self.setP2(value)



# -*- coding: utf-8 -*-

from typing import Optional

from qtpy import QtWidgets

from prettyqt import gui, widgets


class SingleLineTextEdit(widgets.PlainTextEdit):
    def __init__(self, parent: Optional[QtWidgets.QWidget] = None):
        super().__init__(parent=parent)
        self.textChanged.connect(self._on_text_changed)
        font_metrics = gui.FontMetrics(self.font())
        self.row_height = font_metrics.lineSpacing()
        self.setFixedHeight(self.row_height * 1.5)
        self.set_size_policy(vertical="fixed")
        self.set_line_wrap_mode("none")
        self.set_scrollbar_policy("always_off")

    def _on_text_changed(self):
        text = self.text()
        with self.current_cursor() as c:
            pos = c.position()
            num_linebreaks = text.count("\n")
            with self.block_signals():
                self.set_text(text.replace("\n", ""))
            c.setPosition(pos - num_linebreaks)


if __name__ == "__main__":
    app = widgets.app()
    widget = SingleLineTextEdit()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QGraphicsOpacityEffect.__bases__ = (widgets.GraphicsEffect,)


class GraphicsOpacityEffect(QtWidgets.QGraphicsOpacityEffect):

    pass



# -*- coding: utf-8 -*-

from qtpy import QtWidgets


STYLES = dict(
    close=QtWidgets.QStyle.SP_TitleBarCloseButton,
    maximise=QtWidgets.QStyle.SP_TitleBarMaxButton,
    information=QtWidgets.QStyle.SP_MessageBoxInformation,
    warning=QtWidgets.QStyle.SP_MessageBoxWarning,
    critical=QtWidgets.QStyle.SP_MessageBoxCritical,
    question=QtWidgets.QStyle.SP_MessageBoxQuestion,
)


class Style(QtWidgets.QStyle):
    pass



# -*- coding: utf-8 -*-

from qtpy import QtCore, QtWidgets

from prettyqt import gui, core
from prettyqt.utils import bidict, InvalidParamError


STATES = bidict(
    unchecked=QtCore.Qt.Unchecked,
    partial=QtCore.Qt.PartiallyChecked,
    checked=QtCore.Qt.Checked,
)


class TreeWidgetItem(QtWidgets.QTreeWidgetItem):
    def __repr__(self):
        return "TreeWidgetItem()"

    def serialize_fields(self):
        icon = self.icon(0)
        return dict(
            text=self.text(0),
            tool_tip=self.toolTip(0),
            status_tip=self.statusTip(0),
            checkstate=self.get_checkstate(),
            icon=gui.Icon(icon) if not icon.isNull() else None,
            data=self.data(0, QtCore.Qt.UserRole),
        )

    def __getstate__(self):
        return core.DataStream.create_bytearray(self)

    def __setstate__(self, ba):
        self.__init__()
        core.DataStream.write_bytearray(ba, self)

    def set_icon(self, icon: gui.icon.IconType):
        """Set the icon for the action.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon)
        self.setIcon(0, icon)

    def set_checkstate(self, state: str):
        """Set checkstate of the checkbox.

        valid values are: unchecked, partial, checked

        Args:
            state: checkstate to use

        Raises:
            InvalidParamError: invalid checkstate
        """
        if state not in STATES:
            raise InvalidParamError(state, STATES)
        self.setCheckState(0, STATES[state])

    def get_checkstate(self) -> str:
        """Return checkstate.

        possible values are "unchecked", "partial", "checked"

        Returns:
            checkstate
        """
        return STATES.inv[self.checkState(0)]


if __name__ == "__main__":
    item = TreeWidgetItem()
    item.setData(0, 1000, "test")



# -*- coding: utf-8 -*-

from typing import List

from qtpy import QtWidgets, QtCore, QtGui

from prettyqt.utils import bidict, InvalidParamError

ITEM_SELECTION_MODES = bidict(
    contains_shape=QtCore.Qt.ContainsItemShape,
    intersects_shape=QtCore.Qt.IntersectsItemShape,
    contains_bounding_rect=QtCore.Qt.ContainsItemBoundingRect,
    intersects_bounding_rect=QtCore.Qt.IntersectsItemBoundingRect,
)

FOCUS_REASONS = bidict(
    mouse=QtCore.Qt.MouseFocusReason,
    tab=QtCore.Qt.TabFocusReason,
    backtab=QtCore.Qt.BacktabFocusReason,
    active_window=QtCore.Qt.ActiveWindowFocusReason,
    popup=QtCore.Qt.PopupFocusReason,
    shortcut=QtCore.Qt.ShortcutFocusReason,
    menu_bar=QtCore.Qt.MenuBarFocusReason,
    other=QtCore.Qt.OtherFocusReason,
)

MODALITIES = bidict(
    none=QtWidgets.QGraphicsItem.NonModal,
    panel=QtWidgets.QGraphicsItem.PanelModal,
    scene=QtWidgets.QGraphicsItem.SceneModal,
)


class GraphicsItem(QtWidgets.QGraphicsItem):
    def __contains__(self, value: QtCore.QPointF) -> bool:
        return self.contains(value)

    def set_focus(self, reason: str = "other"):
        if reason not in FOCUS_REASONS:
            raise InvalidParamError(reason, FOCUS_REASONS)
        self.setFocus(FOCUS_REASONS[reason])

    def colliding_items(self, mode: str = "intersects") -> List[QtWidgets.QGraphicsItem]:
        if mode not in ITEM_SELECTION_MODES:
            raise InvalidParamError(mode, ITEM_SELECTION_MODES)
        return self.collidingItems(ITEM_SELECTION_MODES[mode])

    def collides_with(self, item: QtGui.QPainterPath, mode: str = "intersects") -> bool:
        if mode not in ITEM_SELECTION_MODES:
            raise InvalidParamError(mode, ITEM_SELECTION_MODES)
        if isinstance(item, QtGui.QPainterPath):
            return self.collidesWithPath(item, ITEM_SELECTION_MODES[mode])
        else:
            return self.collidesWithItem(item, ITEM_SELECTION_MODES[mode])

    def set_panel_modality(self, modality: str = "window") -> None:
        """Set panel modality.

        Valid values for modality: "none", "panel", "scene"

        Args:
            modality: panel modality

        Raises:
            InvalidParamError: panel modality does not exist
        """
        if modality not in MODALITIES:
            raise InvalidParamError(modality, MODALITIES)
        self.setPanelModality(MODALITIES[modality])

    def get_panel_modality(self) -> str:
        """Get the current modality modes as a string.

        Possible values: "none", "panel", "scene"

        Returns:
            panel modality
            str
        """
        return MODALITIES.inv[self.panelModality()]
